PAGE_SIZE = 4096
CACHE_SIZE = 2000
DEFAULT_COMPRESSION_LEVEL = 6
MAX_COMPRESSION_LEVEL = 9
MIN_COMPRESSION_LEVEL = 0

MAGIC_NUMBER = b'RBXDB'
VERSION = 1

BTREE_ORDER = 128
MAX_PAGE_CACHE = 1000

WAL_MAGIC = b'RBXWAL'
CHECKPOINT_INTERVAL = 1000

HASH_ALGORITHM = 'blake2b'
HASH_SIZE = 32
